@@ -5,13 +5,14 @@
 % FORMAT   ret = arts_nc_read_tensor4(filename)
 %        
 % OUT   ret      Tensor4
+% OUT   gattr    Global attributes
 %       
 % IN    filename Name of NetCDF input file.
 
 % 2010-02-02   Created by Oliver Lemke.
 
-function ret = arts_nc_read_tensor4 (filename)
+function [ret, gattr] = arts_nc_read_tensor4 (filename)
 
-v = loadncvar (filename, 'Tensor4');
+[v, gattr] = loadncvar (filename, 'Tensor4', true);
 d = size(v);
 ret = permute (reshape (v, [d(2) d(3) d(4) d(1)]), [4 3 2 1]);
