@@ -1,17 +1,18 @@
-function pspoly(in)
-%% my_psbox
-% purpose:
-%         Makes boxes
+function commands = pspoly(file,in,commands)
+% PSPOLY Make polygons to plot with GMT
 %
+% IN      file     %s       file to append to
+%         in       struct   options here (see help gmt_plot)
+%         command  cell     cell of string commands for systemcall
 %
-% Author:Salomon Eliasson
-%%
+% OUT     command  cell     cell of string commands for systemcall
+%
+% Created by Oliver Lemke
+% $Id: pspoly.m 7157 2011-09-18 10:38:57Z seliasson $
 
 colors = in.plot.pspoly.colors;
 thick = in.plot.pspoly.thick;
 pp = in.plot.pspoly.pspoly;
-region = in.plot.region;
-proj = in.plot.coast.proj;
 
 for i = 1:size(pp,1)    
     %stdin
@@ -30,18 +31,13 @@ for i = 1:size(pp,1)
     stdin = [stdin,a{size(ppcur,1)+1},' D'];
     
     command = sprintf('echo -e "%s" |',stdin);
-    command = sprintf('%s psxy -R%s -J%s -A',command,region,proj);
-    
-    if ~isempty(dir(sprintf('%s.ps',in.filename)))
-        command = sprintf('%s -O',command);
-    end
+    command = sprintf('%s psxy -R -J -A',command);
+    command = sprintf('%s %s',command,in.plotPlacement);
     
     % color and size of box
     
     c = colors{i};
     s = thick{i};
     command = sprintf('%s -W%d/%d/%d/%d',command,s,c(1),c(2),c(3));
-    command = sprintf('%s -K>> %s.ps',command,in.filename);
-
-    exec_gmt_cmd(command)
-end
+    commands{end+1} = sprintf('%s -K -O >> %s.ps',command,file);
+end
\ No newline at end of file
