@@ -1,63 +1,49 @@
 % OEM   Data inversion by "optimal estimation method"
 %
-%    The name OEM is here kept as it is so well established in the atmospheric
-%    community. A better name would be "the maximum a posteriori solution". See
-%    "Inverse methods for atmospheric sounding: Theory and practice" by C.D.
-%    Rodgers for background and theory.
+%   The name OEM is here kept as it is so well established in the atmospheric
+%   community. A better name would be "the maximum a posteriori solution". See
+%   "Inverse methods for atmospheric sounding: Theory and practice" by C.D.
+%   Rodgers for background and theory.
 %
-%    The Marquardt-Levenberg method is implemented following Eq. 5.36 of
-%    Rodgers' book. The gamma factor, here denoted as ga, is not allowed to
-%    exceed O.ga_max. When a succesful iteration has been performed, ga is
-%    decreased following O.ga_factor_ok. For cases when a lower cost is not
-%    obtained, ga is increased by O.ga_factor_not_ok. Values of ga below 1 will
-%    be treated as zero. If not a lower cost is obtained for ga=0, then ga=1 is
-%    tested. O.ga_start is used as start value for ga.
+%   The OEM calculations are controlled by the structure O. The fields of O,
+%   and associated descriptions, are listed by
+%      qinfo(@oem)
 %
-%    The OEM calculations are controlled by the structure O. The fields of O,
-%    and associated descriptions, are listed by
-%       qinfo(@oem)
+%   The Marquardt-Levenberg method is implemented following Eq. 5.36 of
+%   Rodgers' book. The gamma factor, here denoted as ga, is not allowed to
+%   exceed O.ga_max. When a succesful iteration has been performed, ga is
+%   decreased following O.ga_factor_ok. For cases when a lower cost is not
+%   obtained, ga is increased by O.ga_factor_not_ok. Values of ga below 1 will
+%   be treated as zero. If not a lower cost is obtained for ga=0, then ga=1 is
+%   tested. O.ga_start is used as start value for ga.
 %
-%    The communication with the forward model is handled by the function
-%    handle comfun. If comfun is set to @arts_oem, the following calls will 
-%    be made:
-%       [R,yf,J]  = arts_oem( Q, R, x, iter );
-%       [R,yf]    = arts_oem( Q, R, x, iter );
-%    That is, this function expects the comfun function to provide spectrum 
-%    and jacobian (only spectrum in the second case) for present state (the x 
-%    vector). The oem function does not use any information in Q and R, and
-%    the comfun function is free to use these variables freely for bookkeeping
-%    etc. This function handles y, x, Sx and Se as totally generic variables
-%    and unit conversion and similar tasks are left to the comfun and
-%    post-processing. This with two exceptions, jacobian rescaling is
-%    performed following the O-fields i_rel and i_log (see further information
-%    for these fields).
-%
-%    The results are gathered in a structure X, with fields:
-%       x         : Retrieved state.
-%       converged : Convergence flag. 1 if convergency criterion has been 
-%                   reached. Otherwise 0 for non-linear inversions when the
-%                   maximum number of iterations is reached, or -1 for 
-%                   Marquardt-Levenberg when the maximum value for gamma is
-%                   passed. Set to NaN for linear retrievals.
-%       cost      : Total cost, normalised by length of y. A succesful retrieval
-%                   shall then have a cost in the order of 1.
-%       cost_y    : Cost corresponding to the measurement vector.
-%       cost_x    : Cost corresponding to the state vector.
-%       dx        : Change in the state vector x between each iteration.
-%                   A normalisation to the length of x is applied.
-%       e         : Total retrieval error (square root of diagonal elements 
-%                   of S).
-%       eo        : Observation error (square root of diagonal elements of So).
-%       es        : Smoothing error (square root of diagonal elements of Ss).
-%       yf        : Fitted spectrum.
-%       A         : Averaging kernel matrix.
-%       G         : Gain matrix (also denoted as Dy).
-%       J         : Jacobian (weighting function matrix).
-%       S         : Covariance matrix for total error. That is, the sum
-%                   of So and Ss.
-%       So        : Covariance matrix for observation error.
-%       Ss        : Covariance matrix for smoothing error.
-%       Xiter     : The x-vector after each iteration.
+%   The results are gathered in a structure X, with fields:
+%      x         : Retrieved state.
+%      converged : Convergence flag. 1 if convergency criterion has been 
+%                  reached. Otherwise 0 for non-linear inversions when the
+%                  maximum number of iterations is reached, or -1 for 
+%                  Marquardt-Levenberg when the maximum value for gamma is
+%                  passed. Set to NaN for linear retrievals.
+%      cost      : Total cost, normalised by length of y. A succesful retrieval
+%                  shall then have a cost in the order of 1.
+%      cost_y    : Cost corresponding to the measurement vector.
+%      cost_x    : Cost corresponding to the state vector.
+%      dx        : Change in the state vector x between each iteration.
+%                  A normalisation to the length of x is applied.
+%      e         : Total retrieval error (square root of diagonal elements 
+%                  of S).
+%      eo        : Observation error (square root of diagonal elements of So).
+%      es        : Smoothing error (square root of diagonal elements of Ss).
+%      ex        : A priori uncertainty (square root of diag elements of Sx).
+%      yf        : Fitted spectrum.
+%      A         : Averaging kernel matrix.
+%      G         : Gain matrix (also denoted as Dy).
+%      J         : Jacobian (weighting function matrix).
+%      S         : Covariance matrix for total error. That is, the sum
+%                  of So and Ss.
+%      So        : Covariance matrix for observation error.
+%      Ss        : Covariance matrix for smoothing error.
+%      Xiter     : The x-vector after each iteration.
 %   The fields *x* and *converged* are always included (if an actual inversion 
 %   is performed). Other fields are included if corresponding O field is set.
 %   Note that the case x=x0, where x0 is the start value of x, is treated as
@@ -65,6 +51,19 @@
 %   corresponds to the sitatuation when no inversion/iteration has been
 %   performed. 
 %
+%   The communication with the forward model is handled by the function
+%   handle comfun. If comfun is set to @arts_oem, the following calls will 
+%   be made:
+%      [R,yf,J]  = arts_oem( Q, R, x, iter );
+%      [R,yf]    = arts_oem( Q, R, x, iter );
+%   That is, this function expects the comfun function to provide spectrum 
+%   and jacobian (only spectrum in the second case) for present state (the x 
+%   vector). The oem function does not use any information in Q and R, and
+%   the comfun function is free to use these variables freely for bookkeeping
+%   etc. This function handles y, x, Sx and Se as totally generic variables
+%   and unit conversion and similar tasks are left to the comfun and
+%   post-processing. 
+%
 %   Use the first format to check default values for O. The second format
 %   is intended for extracting retrieval characteristics (corresponding to
 %   a linear inversion). The third format performs a complete inversion.
@@ -137,6 +136,12 @@ I.es = [ ...
 'diagonal elements of Ss).'
 ];
 %-----------------------------------------------------------------------------
+O.ex = false;
+I.ex = [ ...
+'Flag to include in X the a priori uncertainty (square root of ',...
+'diagonal elements of Sx).'
+];
+%-----------------------------------------------------------------------------
 O.G = false;
 I.G = [ ...
 'Flag to include gain matrix in X (alse denoted as Dy).'
@@ -175,28 +180,6 @@ I.itermethod = [ ...
 '''LM'' for Marquardt-Levenberg.'
 ];
 %-----------------------------------------------------------------------------
-O.i_log = [];
-I.i_log = [ ...
-'Index of values in x that are retrieved in a log-scale (e.g. the logarithm ',...
-'of VMR/VMRa (see i_rel)). The actual inversion is performed with jacobians ',...
-'provided (that is, must be given to oem.m scaled correctly), but the ',...
-'log-scale transformation is removed for the error characterisation for ',...
-'the jacobians matching this field. No index should be found in both i_rel ',...
-'and i_log.'
-];
-%-----------------------------------------------------------------------------
-O.i_rel = [];
-I.i_rel = [ ...
-'Index of values in x that are retrieved in a relative/fractional unit and a',...
-'rescaling of the jacobians is needed. An example on a relative retrieval ',... 
-'is x = VMR/VMRa, where VMRa is the volume mixing ratio for the a priori ',...
-'state. Accordingly, the corresponding jacobians must describe changes with ',...
-'respect to the a priori state, and a direct calculation for x != xa will ',...
-'not give a correct results. A rescaling back to xa is then needed and this ',...
-'operation can be performed by oem.m by setting this field to the indexes ',...
-'for which a rescaling is needed.'
-];
-%-----------------------------------------------------------------------------
 O.J = false;
 I.J = [ ...
 'Flag to include weighting function matrix in X.'
@@ -267,9 +250,9 @@ I.stop_dx = [ ...
 O.sxnorm = false;
 I.sxnorm = [ ...
 'Flag to internally perform a normalisation of x, based on the diagonal ',...
-'elements of Sx. This to avoid a pure condition number for a matrix ',...
-'inversion that is encountered when units are such that retrieved values ',...
-'vary strongly in magnitude.',...
+'elements of Sx. Numerical problems can occur when the retrieved values ',...
+'differ strongly in magnitude (due to poor condition number for matrix ',...
+'inversions). This flag can be used to overcome this problem.',...
 '#The inverse of Sx must be calculated internally if this option is used ',...
 'and there is no use in pre-calculating *Sxinv*.'
 ];
@@ -344,9 +327,6 @@ end
     end                                                                     %&%
   end                                                                       %&%
   %                                                                         %&%
-  if ~isempty( intersect( O.i_log, O.i_rel ) )                              %&%
-    error( 'No index can be part of both O.i_rel and O.i_log.' );           %&%
-  end                                                                       %&%
                                                                             %&%
   rqre_datatype( Q, @isstruct );                                            %&%
   rqre_datatype( R, {@isempty,@isstruct} );                                 %&%
@@ -485,9 +465,9 @@ end
   else
     
     % The different cases are handled separately to avoid a lot of flags, if
-    % statements and control variables. Helps also to save memory and to make the
-    % calculations in most efficient way. The drawback is that more or less the
-    % same code is found at several places.
+    % statements and control variables. Helps also to save memory and to make
+    % the calculations in most efficient way. The drawback is that more or less
+    % the same code is found at several places.
     
 
     
@@ -741,6 +721,8 @@ end
                           test_convergence( x, xnew, SJSJ, O, xnorm, X, iter );
           nlinprint( O.outfids, iter, ga, cost, cost_x, cost_y, dx );
           x = xnew;
+        else
+          break;
         end
       end % while ~converged
       
@@ -767,13 +749,9 @@ end
   %
   if O.J | O.G | O.A | O.S | O.So | O.Ss | O.e | O.eo | O.es    
     %
-    if isnan(J)  |  ( O.jexact  &  ~j_updated )  |  ~isempty( O.i_log )
+    if isnan(J)  |  ( O.jexact  &  ~j_updated ) 
       [R,yf,J] = calc_y_and_j( comfun, Q, R, O, xnorm, x, iter );
       %
-      if ~isempty( O.i_log )
-        J(:,O.i_log) =  J(:,O.i_log) ./ repmat( exp(x(O.i_log)'), size(J,1), 1 );
-      end
-      %
       JtSeinv  = J' * Seinv;
       SJSJ     = Sxinv + JtSeinv * J;
     end
@@ -821,7 +799,7 @@ end
 
   if O.J
     if O.sxnorm
-      X.J = J ./ repmat( xnorm', ny , 1 );
+      X.J = J ./ repmat( xnorm', size(J,1) , 1 );
     else
       X.J = J;
     end
@@ -829,14 +807,18 @@ end
 
   if O.G
     if O.sxnorm
-      X.G = G .* repmat( xnorm', ny , 1 );
+      X.G = G .* repmat( xnorm, 1, size(G,2) );
     else
       X.G = G;
     end
   end
 
   if O.A 
-    X.A = A;  % No normalisation needed here
+    if O.sxnorm
+      X.A = A .* (xnorm*(1./xnorm'));  %scaling is ni/nj, where ni is xnorm
+    else                               %for retrieved value, and nj for x-state
+      X.A = A; 
+    end
   end  
 
   if O.S  |  O.So  |  O.e  |  O.eo
@@ -858,6 +840,7 @@ end
   if O.S  |  O.Ss  |  O.e  |  O.es
     AI = A - eye(size(A,1));
     Ss = AI * Sx * AI';
+    clear AI;
     %
     if O.sxnorm
       Ss = Ss .* (xnorm*xnorm');
@@ -880,6 +863,13 @@ end
     end
   end  
 
+  if O.ex
+    if O.sxnorm
+      X.ex = full( sqrt( diag( Sx ) ) .* xnorm );
+    else
+      X.ex = full( sqrt( diag( Sx ) ) );
+    end
+  end
 %^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
 
@@ -962,10 +952,6 @@ end
     if O.sxnorm
       J = J .* repmat( xnorm', size(J,1), 1 );
     end
-    %
-    if ~isempty( O.i_rel )
-      J(:,O.i_rel) =  J(:,O.i_rel) ./ repmat( x(O.i_rel)', size(J,1), 1 );
-    end
   return
 %^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
